<?php

require_once 'base/EstruturamEntityBase.php';

/**
 * EstruturamEntity
 *
 * @package    Geral
 * @subpackage Entity
 * @author     alessandrobartels
 */
class EstruturamEntity extends EstruturamEntityBase
{
    private $_nrOrg;
    private $_mesCompetencia;

    private $_nrEstruturaAnterior;
    private $_estrutura;
    /** @var $historico EstruturahEntity */
    public $historico;

    public function __construct($id = null, $nrOrg = null, $mesCompetencia = null) {
        parent::__construct($id);
        $this->_nrOrg = $nrOrg;
        $this->_mesCompetencia = $mesCompetencia;
    }


    public function getEstrutura(){
        return $this->_estrutura;
    }

    public function insereEstrutura($nrorg, $nrtipoestrutura, $cdintestrutura, $dtinivigencia) {
        $this->setNrorg($nrorg);
        $this->setNrtipoestrutura($nrtipoestrutura);
        $this->setCdintestrutura($cdintestrutura);
        $this->setDtinivigencia($dtinivigencia);
    }

    public function carregaEstrutura($nrEstrutura){

        try {
            if ($this->_nrEstruturaAnterior !== $nrEstrutura) {
                // Procura na lista e retorna a posicao.
                $posicao = $this->_procuraEstrutura($nrEstrutura);
                if ($posicao === -1) {

                    // Procura na base caso nao exista na lista.
                /*    $this->_paramEstrutura = GpeParamestrutm::retornaParametrosEstrutra(
                        $this->_nrOrg, $this->_mesCompetencia, $nrEstrutura
                    );
                 */
                    $this->_estrutura = $this->retornaEstrutura($nrEstrutura);


                    $this->addItem($this->_estrutura);
                    $this->_nrEstruturaAnterior = $nrEstrutura;
                } else {

                    $this->_estrutura = $this->getItem($posicao);
                }
            }
        } catch (Exception $e){
            throw  $e;
        }

    }


    private function _procuraEstrutura($nrEstrutura) {

        $itens = $this->getItems();

        foreach ($itens as $key=>$item) {
            if (($item->getNrorg() === $this->_nrOrg) && ($item->getNrestruturam() === $nrEstrutura)) {
                return $key;
            }
        }

        return -1;
    }



    /**
     * Retorna estrutura.
     *
     * @param integer $nrEstrutura Numero da estrutura.
     *
     * @return EstruturamEntity
     */
    public function retornaEstrutura($nrEstrutura){
        $estrutura = Estruturam::retornaEstrutura($this->_nrOrg, $nrEstrutura, $this->_mesCompetencia);
        if (is_object($estrutura) === true) {

            // Retorna o historico mais atual, de acordo com a data.
            $historico = end(Estruturah::retornaEstruturaHistorico($this->_nrOrg, $nrEstrutura, $this->_mesCompetencia));

            $estrutura->historico = $historico;
            return $estrutura;
        } else {
            return null;
        }
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrestruturam    Nr Estrutura.
     * @param integer $nrtipoestrutura Nr Tipo Estrutura.
     * @param integer $nrorgTrab       Nr Org.
     *
     * @return boolean
     *
     * @author luistavares 12/08/2011
     */
    public function novoMestre($nrestruturam, $nrtipoestrutura, $nrorgTrab){
        try{
          //  $now = new TkDateTime();
          //  $now = $now->toString(TkDateTime::FORMAT_DATE);

            $this->setNrestruturam($nrestruturam);
            $this->setNrtipoestrutura($nrtipoestrutura);
            $this->setDtinivigencia($now);
            $this->setNrorg($nrorgTrab);
         //   $this->setDtinclusao($now);

            $this->persist();

            return true;
        }
        catch (Exception $e){
            return false;
        }

    }


    /**
     * Enter description here ...
     *
     * @param integer $nrestruturam Nr Estrutura.
     * @param integer $nrorgTrab    Nr Org.
     *
     * @return boolean
     *
     * @author luistavares 12/08/2011
     */
    public function destroiMestre($nrestruturam, $nrorgTrab){
        try{
            $estruturam = Estruturam::retornaEstruturaPelaChave($nrestruturam, $nrorgTrab);
            if (is_object($estruturam) === true) {
                /* Adiciona um item � lista, ap�s isso, o item � removido da lista
                 * e enviado para lista de itens removidos, lista essa que ir� apagar
                 * todos seus itens quando executar o persist(). */
                $this->addItem($estruturam, $nrestruturam);
                $this->removeItem($estruturam);
                $this->persist();

                return true;
            } else {
                return false;
            }
        }
        catch (Exception $e){
            tk_warning($e->getMessage());
            return false;
        }

    }


    public function criaEstruturaMestre($nrOrg, $dtIniVigencia, $tipoEstrutura, $cdInterno){

        $estrutura = new self();

        $estrutura->setNrorg($nrOrg);
        $estrutura->setDtinivigencia($dtIniVigencia);
        $estrutura->setNrtipoestrutura($tipoEstrutura);
        $estrutura->setCdintestrutura($cdInterno);

        $estrutura->persist();

        // Guarda o valor do id da estruturam para gerar o historico
        $this->setNrestruturam($estrutura->getNrestruturam());

        unset($estrutura);

    }


    public function retornaEstruturaPorCdIntegracao($cdIntegracao){
        $resultado = Estruturam::retornaEstruturaPorCdIntegracao($this->_nrOrg, $cdIntegracao);
        return $resultado;
    }

    /**
     * @param $nrorg
     * @param $cdintestrutura
     *
     * @return EstruturamEntity
     */
    public static function retornaEstruturaPorCdintestrutura($nrorg, $cdintestrutura) {
        $estruturam = self::getEntityByWhere( 'NRORG = ? AND CDINTESTRUTURA = ?', array($nrorg, $cdintestrutura));
        return $estruturam;
    }

    public static function retornaEstruturaPorNrestruturam($nrorg, $nrestruturam) {
        $estruturam = self::getEntityByWhere( 'NRORG = ? AND NRESTRUTURAM = ?', array($nrorg, $nrestruturam));
        return $estruturam;
    }

    /**
     * @param $nrorg
     * @param $cdintestrutura
     * @param $nrtipoestrutura
     *
     * @return EstruturamEntity
     */
    public static function retornaEstruturaPorCdintestruturaETipo($nrorg, $cdintestrutura, $nrtipoestrutura) {
        $estruturam = self::getEntityByWhere( 'NRORG = ? AND CDINTESTRUTURA = ? AND NRTIPOESTRUTURA = ?', array($nrorg, $cdintestrutura, $nrtipoestrutura));
        return $estruturam;
    }

    public function getHistorico() {
        if (!$this->historico) {
            $this->historico = EstruturahEntity::getEntityByWhere(
                'NRORG = ? AND NRESTRUTURAM = ?',
                array($this->getNrorg(), $this->getNrestruturam()),
                //'NRESTRUTURAH DESC'
                'DTMESCOMPETENC DESC'
            );
        }
        return $this->historico;
    }

    /**
     * @param null $nrTipoHierarquiaTomador
     * @return EstruturamEntity | null
     */
    public function getEstruturaSuperior($nrTipoHierarquiaTomador = null) {
        /** @var $hierarqorg HierarquiaorgEntity */
        if (!empty($nrTipoHierarquiaTomador)) {
            $hierarqorg = HierarquiaorgEntity::getEntityByWhere(
                'NRORG = ? AND NRESTRUTURAMREL = ? AND NRTIPOHIERARQORG = ?', array($this->getNrorg(), $this->getNrestruturam(), $nrTipoHierarquiaTomador)
            );
        } else {
            $hierarqorg = HierarquiaorgEntity::getEntityByWhere(
                'NRORG = ? AND NRESTRUTURAMREL = ?', array($this->getNrorg(), $this->getNrestruturam())
            );
        }

        if (!$hierarqorg) {
            return null;
        }

        /** @var $estruturaSuperior EstruturamEntity */
        $estruturaSuperior = self::getEntityByWhere(
            'NRORG = ? AND NRESTRUTURAM = ?',
            array($this->getNrorg(), $hierarqorg->getNrestruturamsup())
        );
        return $estruturaSuperior;
    }


}